//
// Copyright 2021, Jesús Pérez Lorenzo
//
use serde::{Deserialize};

#[derive(Clone, Debug, Deserialize)]
pub struct Notifier {
  pub name: String,
  pub settings: String,
  pub command: String,
}

#[derive(Clone, Debug, Deserialize, Default)]
pub struct StoreSettings {
  host: String,
  port: u32,
  user: String,
  pass: String,
  pub database: String,
  pub max_conn: u32,
}
impl StoreSettings {
  #[must_use]
  #[allow(unused_variables)]
  pub fn url(&self, store: &str, key: &str) -> String {
    let mut user_pass = String::from("");
    let user: String;
    let pass: String;
    user = self.user.to_owned();
    pass = self.pass.to_owned();
    if user.as_str() != "" || pass.as_str() != "" {
      user_pass = format!("{}:{}", &user, &pass);
    }
    let host = &self.host;
    let port = &self.port;
    let database = &self.database;
    format!(
      "{}://{}@{}:{}/{}",
      store, &user_pass, &host, &port, &database
    )
  }
}
#[derive(Clone, Debug, Deserialize, Default)]
pub struct StoreKeyValue {
  host: String,
  port: u32,
  pub prefix: String,
  pub max_conn: u32,
}
impl StoreKeyValue {
  #[must_use]
  pub fn url(&self, store: &str) -> String {
    let host = &self.host;
    let port = &self.port;
    format!("{}://{}:{}", store, &host, &port)
  }
}
#[derive(Clone, Debug, Deserialize, Default)]
pub struct StoreLocal {
  pub database: String,
}
impl StoreLocal {
  #[must_use]
  pub fn url(&self, store: &str) -> String {
    format!("{}.{}", store, &self.database)
  }
}
#[derive(Clone, Debug, Deserialize, Default)]
pub struct WebServer {
  pub name: String,
	pub home_path: String,
	pub resources_path: String,

  pub certs_store_path: String,
  pub cert_file_sufix: String,

  pub default_module: String,
  // Some paths 
  pub templates_path: String,
  pub defaults_path: String,
  pub dist_path: String,
  pub html_path: String,
  pub upload_path: String,
  /// allow origin for localhost 
  pub allow_origin: Vec<String>,
	/// graphql schemas
  pub gql_schemas_path: String,
  /// graphql query targets
  pub gql_targets: String,
  /// graphql path for requests
  pub gql_req_path: String, 
  /// graphQL path for request
  pub giql_req_path: String,
  pub auth_model_path: String,
  pub auth_policy_path: String,
  pub usrs_store: String,
  pub usrs_store_target: String,
  pub usrs_shadow_store: String,
  pub usrs_shadow_target: String,
  pub main_store: String,
  pub srv_host: String,
  pub srv_port: u16,
  pub srv_protocol: String,
	pub langs: Vec<String>,
  pub signup_mode: String,
  pub password_rules: String,
  pub mapped_url_prefix: String,
  pub admin_key: String,
}
impl WebServer {
  #[must_use]
  pub fn st_html_path(&self) -> &'static str  {
    Box::leak(self.html_path.to_owned().into_boxed_str())
  }
  #[must_use]
  pub fn st_auth_model_path(&self) -> &'static str  {
    Box::leak(self.auth_model_path.to_owned().into_boxed_str())
  }
  #[must_use]
  pub fn st_auth_policy_path(&self) -> &'static str  {
    Box::leak(self.auth_policy_path.to_owned().into_boxed_str())
  }
  #[must_use]
  pub fn st_gql_req_path(&self) -> &'static str  {
    Box::leak(self.gql_req_path.to_owned().into_boxed_str())
  }
  #[must_use]
  pub fn st_giql_req_path(&self) -> &'static str  {
    Box::leak(self.giql_req_path.to_owned().into_boxed_str())
  }
}

#[derive(Clone, Debug, Deserialize, Default)]
pub struct Config {
  pub run_mode: String,
  // warp log name
  pub log_name: String,
  pub loop_duration: u64,
  pub run_cache: bool,
  pub cache_path: String,
  pub cache_lock_path: String,
  pub cache_lock_ext: String,
  pub run_check: bool,
  pub check_path: String,
  pub default_lang: String,
  pub logs_store: String,
  pub logs_format: String,
  pub websrvrs: Vec<WebServer>,
}

impl Config {
  pub fn load_file_content(verbose: &str, cfg_path: &str) -> String {
    let config_path: String;
		if cfg_path.is_empty() {
		 	config_path = envmnt::get_or("APP_CONFIG_PATH", "config.toml");
		} else {
		  config_path = cfg_path.to_string();
		}
		if verbose != "quiet" {
    	println!("Config path: {}", &config_path);
		}
    std::fs::read_to_string(&config_path)
      .unwrap_or_else(|e| {
        eprintln!("{}",e);
        String::from("")
    })
  }
  pub fn new(content: String,verbose: &str) -> Self { 
    match toml::from_str(&content) {
      Ok(cfg) => {
				if verbose != "quiet" {
          println!("Config Loaded successfully");
				}
				let app_home=envmnt::get_or("APP_HOME", "");
				// if app_home.is_empty() {
        // 	cfg
				// } else {
					let mut app_cfg: Config = cfg;
					let mut websrvrs: Vec<WebServer> = Vec::new();
					app_cfg.websrvrs.iter().enumerate().for_each(|(pos,it)| {
						let app_home_path=format!("{}/{}/",&app_home,&it.home_path);
						websrvrs.push(it.to_owned());
						websrvrs[pos].certs_store_path=format!("{}{}",&app_home_path,&it.certs_store_path);
						websrvrs[pos].resources_path=format!("{}{}",&app_home_path,&it.resources_path);
						websrvrs[pos].templates_path=format!("{}{}",&app_home_path,&it.templates_path);
						websrvrs[pos].defaults_path=format!("{}{}",&app_home_path,&it.defaults_path);
						websrvrs[pos].html_path=format!("{}{}",&app_home_path,&it.html_path);
						websrvrs[pos].dist_path=format!("{}{}",&app_home_path,&it.dist_path);
						websrvrs[pos].upload_path=format!("{}{}",&app_home_path,&it.upload_path);
						websrvrs[pos].auth_model_path=format!("{}{}",&app_home_path,&it.auth_model_path);
						websrvrs[pos].auth_policy_path=format!("{}{}",&app_home_path,&it.auth_policy_path);
						websrvrs[pos].usrs_store_target=format!("{}{}",&app_home_path,&it.usrs_store_target);
						websrvrs[pos].usrs_shadow_target=format!("{}{}",&app_home_path,&it.usrs_shadow_target);
					});
					app_cfg.websrvrs = websrvrs;
					app_cfg.cache_path=format!("{}{}",&app_home,&app_cfg.cache_path);
					app_cfg.cache_lock_path=format!("{}{}",&app_home,&app_cfg.cache_lock_path);
					app_cfg.check_path=format!("{}{}",&app_home,&app_cfg.check_path);
					app_cfg
				// }
      },
      Err(e) => {
        println!("Config error: {}",e);
        Config::default()
      }
    }
  }
	#[must_use]
  pub fn st_log_name(&self) -> &'static str  {
    Box::leak(self.log_name.to_owned().into_boxed_str())
  } 
}
